/* $Id$

This file is part of libmspal
Copyright © 2008 Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#include "buffer.h"
#include "source.h"

using namespace std;

namespace Msp {
namespace AL {

Source::Source()
{
	alGenSources(1, &id);
}

Source::~Source()
{
	alDeleteSources(1, &id);
}

void Source::attribute(ALenum attr, int v)
{
	alSourcei(id, attr, v);
}

void Source::attribute(ALenum attr, float v)
{
	alSourcef(id, attr, v);
}

void Source::attribute(ALenum attr, float v0, float v1, float v2)
{
	alSource3f(id, attr, v0, v1, v2);
}

void Source::attribute(ALenum attr, const float *v)
{
	alSourcefv(id, attr, v);
}

void Source::get_attr(ALenum attr, int *v) const
{
	alGetSourcei(id, attr, v);
}

void Source::get_attr(ALenum attr, float *v) const
{
	alGetSourcef(id, attr, v);
}

SourceState Source::get_state() const
{
	int state;
	get_attr(AL_SOURCE_STATE, &state);
	return static_cast<SourceState>(state);
}

void Source::set_position(float x, float y, float z)
{
	attribute(AL_POSITION, x, y, z);
}

void Source::set_looping(bool l)
{
	attribute(AL_LOOPING, l);
}

void Source::set_buffer(const Buffer &buffer)
{
	attribute(AL_BUFFER, static_cast<int>(buffer.get_id()));
}

void Source::queue_buffers(const vector<const Buffer *> &buffers)
{
	vector<uint> ids;
	ids.reserve(buffers.size());
	for(vector<const Buffer *>::const_iterator i=buffers.begin(); i!=buffers.end(); ++i)
		ids.push_back((*i)->get_id());
	alSourceQueueBuffers(id, ids.size(), &ids.front());
}

void Source::queue_buffer(const Buffer &buffer)
{
	uint bid=buffer.get_id();
	alSourceQueueBuffers(id, 1, &bid);
}

void Source::play()
{
	alSourcePlay(id);
}

void Source::pause()
{
	alSourcePause(id);
}

void Source::stop()
{
	alSourceStop(id);
}

void Source::rewind()
{
	alSourceRewind(id);
}

} // namespace AL
} // namespace Msp
